
package jp.gr.java_conf.u6k.andro_memo.model;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import jp.gr.java_conf.u6k.andro_memo.entity.MemoEntity;
import jp.gr.java_conf.u6k.andro_memo.entity.MemoStateEnum;
import jp.gr.java_conf.u6k.andro_memo.entity.UserEntity;
import jp.gr.java_conf.u6k.andro_memo.util.CollectionUtil;
import jp.gr.java_conf.u6k.andro_memo.util.DateUtil;

import com.google.appengine.api.datastore.Text;

public class MemoModel {

    // private static final Logger LOG = Logger.getLogger(MemoModel.class.getName());

    private PersistenceManager _pm;

    public MemoModel(PersistenceManager pm) {
        this._pm = pm;
    }

    public MemoEntity create(String loginTicketKey, String title, String tag, String content, Date updateDatetime) {
        if (updateDatetime == null) {
            throw new IllegalArgumentException("updateDatetime == null");
        }

        UserModel userModel = new UserModel(this._pm);
        if (!userModel.isLogin(loginTicketKey)) {
            throw new UserNotLoginedException();
        }

        UserEntity user = userModel.findLoginUser(loginTicketKey);

        MemoEntity memo = new MemoEntity();
        memo.setTitle(title);
        memo.setTag(tag);
        if (content != null) {
            memo.setContent(new Text(content));
        }
        memo.setUpdateDatetime(updateDatetime);
        memo.setState(MemoStateEnum.LIVES);
        memo.setUser(user);
        user.getMemoList().add(memo);

        this._pm.makePersistent(user);

        return memo;
    }

    public void update(String loginTicketKey, String memoKey, String title, String tag, String content, Date updateDatetime) {
        if (updateDatetime == null) {
            throw new IllegalArgumentException("updateDatetime == null");
        }

        UserModel userModel = new UserModel(this._pm);
        if (!userModel.isLogin(loginTicketKey)) {
            throw new UserNotLoginedException();
        }

        MemoEntity memo = this.findMemoByKey(memoKey);

        memo.setTitle(title);
        memo.setTag(tag);
        if (content != null) {
            memo.setContent(new Text(content));
        } else {
            memo.setContent(null);
        }
        memo.setUpdateDatetime(updateDatetime);

        this._pm.makePersistent(memo);
    }

    public void delete(String loginTicketKey, String memoKey) {
        UserModel userModel = new UserModel(this._pm);
        if (!userModel.isLogin(loginTicketKey)) {
            throw new UserNotLoginedException();
        }

        MemoEntity memo = this.findMemoByKey(memoKey);

        memo.setState(MemoStateEnum.DELETED);
        memo.setUpdateDatetime(DateUtil.getNow(DateUtil.getUserTimeZone()));

        this._pm.makePersistent(memo);
    }

    public List<MemoEntity> findAll(String loginTicketKey) {
        UserModel userModel = new UserModel(this._pm);
        if (!userModel.isLogin(loginTicketKey)) {
            throw new UserNotLoginedException();
        }

        UserEntity user = userModel.findLoginUser(loginTicketKey);

        Query query = this._pm.newQuery(MemoEntity.class);
        query.setFilter("user == userParam && state == stateParam");
        query.declareParameters("UserEntity userParam, MemoStateEnum stateParam");

        List<MemoEntity> memoList = CollectionUtil.castToList(query.executeWithArray(user, MemoStateEnum.LIVES));
        memoList = new ArrayList<MemoEntity>(this._pm.detachCopyAll(memoList));

        return memoList;
    }

    public MemoEntity findByKey(String loginTicketKey, String memoKey) {
        UserModel userModel = new UserModel(this._pm);
        if (!userModel.isLogin(loginTicketKey)) {
            throw new UserNotLoginedException();
        }

        MemoEntity memo = this.findMemoByKey(memoKey);

        return memo;
    }

    public List<MemoEntity> findByKeyword(String loginTicketKey, String keyword) {
        throw new UnsupportedOperationException(); // TODO
    }

    private MemoEntity findMemoByKey(String key) {
        MemoEntity memo;

        Query query = this._pm.newQuery(MemoEntity.class);
        query.setFilter("key == keyParam && state == stateParam");
        query.declareParameters("String keyParam, MemoStateEnum stateParam");

        List<MemoEntity> memoList = CollectionUtil.castToList(query.execute(key, MemoStateEnum.LIVES));

        if (memoList.size() == 0) {
            memo = null;
        } else {
            memo = memoList.get(0);
        }

        return memo;
    }

}
