
package jp.gr.java_conf.u6k.andro_memo.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public final class HashUtil {

    private static final int BUFFER_LENGTH = 1024;

    private static final int MASK = 0xff;

    private HashUtil() {
        throw new UnsupportedOperationException();
    }

    public static String digest(String algorithm, String src) {
        return HashUtil.digest(algorithm, new ByteArrayInputStream(src.getBytes()));
    }

    public static String digest(String algorithm, InputStream in) {
        try {
            if (in == null) {
                throw new NullPointerException("in == null");
            }

            MessageDigest md = MessageDigest.getInstance(algorithm);

            byte[] buf = new byte[HashUtil.BUFFER_LENGTH];
            int len;
            while ((len = in.read(buf)) != -1) {
                md.update(buf, 0, len);
            }

            byte[] hashData = md.digest();
            String hash = HashUtil.toHexString(hashData);

            return hash;
        } catch (NoSuchAlgorithmException e) {
            throw new HashException(e);
        } catch (IOException e) {
            throw new HashException(e);
        }
    }

    private static String toHexString(byte[] data) {
        StringBuilder hex = new StringBuilder();

        for (int i = 0; i < data.length; i++) {
            String b = Integer.toHexString(data[i] & HashUtil.MASK);
            if (b.length() == 1) {
                hex.append("0");
            }
            hex.append(b);
        }

        return hex.toString();
    }

    @SuppressWarnings("serial")
    public static class HashException extends RuntimeException {

        public HashException(Throwable cause) {
            super(cause);
        }

    }

}
