
package jp.gr.java_conf.u6k.andro_memo.web.json_service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;

import jp.gr.java_conf.u6k.andro_memo.entity.MemoEntity;
import jp.gr.java_conf.u6k.andro_memo.entity.PMF;
import jp.gr.java_conf.u6k.andro_memo.model.MemoModel;

import com.google.appengine.api.datastore.KeyFactory;

public class MemoService {

    public String create(String loginTicketKey, String title, String tag, String content, long updateDatetime) {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            Transaction tx = pm.currentTransaction();
            try {
                tx.begin();

                MemoModel memoModel = new MemoModel(pm);

                MemoEntity memo = memoModel.create(loginTicketKey, title, tag, content, new Date(updateDatetime));

                String key = KeyFactory.keyToString(memo.getKey());

                tx.commit();

                return key;
            } finally {
                if (tx.isActive()) {
                    tx.rollback();
                }
            }
        } finally {
            pm.close();
        }
    }

    public void update(String loginTicketKey, String memoKey, String title, String tag, String content, long updateDatetime) {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            Transaction tx = pm.currentTransaction();
            try {
                tx.begin();

                MemoModel memoModel = new MemoModel(pm);

                memoModel.update(loginTicketKey, memoKey, title, tag, content, new Date(updateDatetime));

                tx.commit();
            } finally {
                if (tx.isActive()) {
                    tx.rollback();
                }
            }
        } finally {
            pm.close();
        }
    }

    public void delete(String loginTicketKey, String memoKey) {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            Transaction tx = pm.currentTransaction();
            try {
                tx.begin();

                MemoModel memoModel = new MemoModel(pm);

                memoModel.delete(loginTicketKey, memoKey);

                tx.commit();
            } finally {
                if (tx.isActive()) {
                    tx.rollback();
                }
            }
        } finally {
            pm.close();
        }
    }

    public List<Map<String, String>> findAll(String loginTicketKey) {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            List<Map<String, String>> resultList = new ArrayList<Map<String, String>>();

            MemoModel memoModel = new MemoModel(pm);

            List<MemoEntity> memoList = memoModel.findAll(loginTicketKey);

            for (MemoEntity memo : memoList) {
                String key = KeyFactory.keyToString(memo.getKey());
                String updateDatetime = Long.toString(memo.getUpdateDatetime().getTime());

                Map<String, String> result = new HashMap<String, String>();
                result.put("key", key);
                result.put("updateDatetime", updateDatetime);

                resultList.add(result);
            }

            return resultList;
        } finally {
            pm.close();
        }
    }

    public Map<String, String> findByKey(String loginTicketKey, String memoKey) {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            MemoModel memoModel = new MemoModel(pm);

            MemoEntity memo = memoModel.findByKey(loginTicketKey, memoKey);

            Map<String, String> result = new HashMap<String, String>();
            result.put("key", KeyFactory.keyToString(memo.getKey()));
            result.put("title", memo.getTitle());
            result.put("tag", memo.getTag());
            if (memo.getContent() != null) {
                result.put("content", memo.getContent().getValue());
            } else {
                result.put("content", null);
            }
            result.put("updateDatetime", Long.toString(memo.getUpdateDatetime().getTime()));

            return result;
        } finally {
            pm.close();
        }
    }

}
