
package jp.gr.java_conf.u6k.andro_memo.web.servlet.html;

import java.io.IOException;

import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import jp.gr.java_conf.u6k.andro_memo.entity.PMF;
import jp.gr.java_conf.u6k.andro_memo.model.UserModel;
import jp.gr.java_conf.u6k.andro_memo.model.UserNotFoundException;

@SuppressWarnings("serial")
public class LoginServlet extends HttpServlet {

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            // パラメータを取得します。
            String email = req.getParameter("email");
            String password = req.getParameter("password");

            if (email == null) {
                email = "";
            } else {
                email = email.trim();
            }

            if (password == null) {
                password = "";
            } else {
                password = password.trim();
            }

            // 入力チェックを行います。
            String message = null;

            if (message == null && "".equals(email)) {
                message = "メールアドレスを入力してください。";
            }
            if (message == null && "".equals(password)) {
                message = "パスワードを入力してください。";
            }

            if (message != null) {
                req.setAttribute("message", message);
                req.setAttribute("email", email);
                req.setAttribute("password", password);

                this.getServletContext().getRequestDispatcher("/html/login.jsp").forward(req, resp);

                return;
            }

            // ログインを行います。
            String loginTicketKey;

            Transaction tx = pm.currentTransaction();
            try {
                tx.begin();

                UserModel userModel = new UserModel(pm);
                loginTicketKey = userModel.login(email, password);

                tx.commit();
            } catch (UserNotFoundException e) {
                loginTicketKey = null;
            } finally {
                if (tx.isActive()) {
                    tx.rollback();
                }
            }

            if (loginTicketKey == null) {
                req.setAttribute("message", "メールアドレス、またはパスワードが間違えています。");
                req.setAttribute("email", email);
                req.setAttribute("password", password);

                this.getServletContext().getRequestDispatcher("/html/login.jsp").forward(req, resp);

                return;
            }

            HttpSession session = req.getSession();
            session.setAttribute("login.loginTicketKey", loginTicketKey);

            // /user/mypage.servletに遷移します。
            resp.sendRedirect(this.getServletContext().getContextPath() + "/html/user/myPage.servlet");
        } finally {
            pm.close();
        }
    }

}
