
package jp.gr.java_conf.u6k.andro_memo.web.servlet.html;

import java.io.IOException;

import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import jp.gr.java_conf.u6k.andro_memo.entity.PMF;
import jp.gr.java_conf.u6k.andro_memo.entity.UserEntity;
import jp.gr.java_conf.u6k.andro_memo.model.UserModel;
import jp.gr.java_conf.u6k.andro_memo.model.UserNotFoundException;

@SuppressWarnings("serial")
public class RegistServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            // パラメータを取得します。
            String key = req.getParameter("key");

            // 仮登録ユーザーを取得します。
            UserModel userModel = new UserModel(pm);
            UserEntity user = userModel.findTempUser(key);

            if (user == null) {
                throw new UserNotFoundException("仮登録状態のユーザーが存在しません。");
            }

            // regist3.jspに遷移します。
            HttpSession session = req.getSession();
            session.setAttribute("regist.key", key);

            req.setAttribute("email", user.getEmail());

            this.getServletContext().getRequestDispatcher("/html/regist3.jsp").forward(req, resp);
        } finally {
            pm.close();
        }
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            // 仮登録ユーザーを取得します。
            HttpSession session = req.getSession();
            String userKey = (String) session.getAttribute("regist.key");

            UserModel userModel = new UserModel(pm);
            UserEntity user = userModel.findTempUser(userKey);

            // パラメータを取得します。
            String password1 = req.getParameter("password1");
            String password2 = req.getParameter("password2");

            if (password1 == null) {
                password1 = "";
            } else {
                password1 = password1.trim();
            }

            if (password2 == null) {
                password2 = "";
            } else {
                password2 = password2.trim();
            }

            String message = null;

            if (message == null && "".equals(password1)) {
                message = "パスワードを入力してください。";
            }
            if (message == null && "".equals(password2)) {
                message = "パスワードを入力してください。";
            }
            if (message == null && !password1.equals(password2)) {
                message = "パスワードは同じ文字を2つ入力してください。";
            }
            if (message == null && !password1.matches("^[0-9a-zA-Z]{7,}$")) {
                message = "パスワードは半角英数字を7文字以上、入力してください。";
            }

            if (message != null) {
                req.setAttribute("message", message);
                req.setAttribute("email", user.getEmail());
                req.setAttribute("password1", password1);
                req.setAttribute("password2", password2);

                this.getServletContext().getRequestDispatcher("/html/regist3.jsp").forward(req, resp);

                return;
            }

            String loginTicketKey;

            Transaction tx = pm.currentTransaction();
            try {
                // 仮登録ユーザーを正式登録します。
                tx.begin();
                userModel.regist(userKey, password1);
                tx.commit();

                // ログインします。
                tx.begin();
                loginTicketKey = userModel.login(user.getEmail(), password1);
                tx.commit();
            } finally {
                if (tx.isActive()) {
                    tx.rollback();
                }
            }

            session.setAttribute("login.loginTicketKey", loginTicketKey);

            // regist4.jspに遷移します。
            this.getServletContext().getRequestDispatcher("/html/regist4.jsp").forward(req, resp);
        } finally {
            pm.close();
        }
    }

}
