
package jp.gr.java_conf.u6k.andro_memo.web.servlet.html.user;

import java.io.IOException;

import javax.jdo.PersistenceManager;
import javax.jdo.Transaction;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import jp.gr.java_conf.u6k.andro_memo.entity.MemoEntity;
import jp.gr.java_conf.u6k.andro_memo.entity.PMF;
import jp.gr.java_conf.u6k.andro_memo.entity.UserEntity;
import jp.gr.java_conf.u6k.andro_memo.model.MemoModel;
import jp.gr.java_conf.u6k.andro_memo.model.UserModel;
import jp.gr.java_conf.u6k.andro_memo.util.DateUtil;

@SuppressWarnings("serial")
public class EditMemoServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            HttpSession session = req.getSession();

            String loginTicketKey = (String) session.getAttribute("login.loginTicketKey");

            String memoKey = req.getParameter("key");

            UserModel userModel = new UserModel(pm);
            UserEntity user = userModel.findLoginUser(loginTicketKey);

            MemoEntity memo;

            if (memoKey != null) {
                MemoModel memoModel = new MemoModel(pm);
                memo = memoModel.findByKey(loginTicketKey, memoKey);
            } else {
                memo = null;
            }

            session.setAttribute("editMemo.memoKey", memoKey);

            req.setAttribute("email", user.getEmail());

            if (memo != null) {
                req.setAttribute("title", memo.getTitle());
                if (memo.getContent() != null) {
                    req.setAttribute("content", memo.getContent().getValue());
                }
                req.setAttribute("updateDatetime", memo.getUpdateDatetime());
            }

            req.getRequestDispatcher("/html/user/editMemo.jsp").forward(req, resp);
        } finally {
            pm.close();
        }
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        PersistenceManager pm = PMF.get().getPersistenceManager();
        try {
            HttpSession session = req.getSession();

            String loginTicketKey = (String) session.getAttribute("login.loginTicketKey");
            String memoKey = (String) session.getAttribute("editMemo.memoKey");

            String memoTitle = req.getParameter("title");
            String memoContent = req.getParameter("content");
            String command = req.getParameter("command");

            if (memoTitle == null) {
                memoTitle = "(タイトルなし)";
            } else if (memoTitle.trim().length() == 0) {
                memoTitle = "(タイトルなし)";
            }

            Transaction tx = pm.currentTransaction();
            try {
                tx.begin();

                if ("保存".equals(command)) {
                    MemoModel memoModel = new MemoModel(pm);

                    if (memoKey != null) {
                        memoModel.update(loginTicketKey, memoKey, memoTitle, null, memoContent, DateUtil.getNow(DateUtil.getUserTimeZone()));
                    } else {
                        memoModel.create(loginTicketKey, memoTitle, null, memoContent, DateUtil.getNow(DateUtil.getUserTimeZone()));
                    }
                } else if ("削除".equals(command)) {
                    MemoModel memoModel = new MemoModel(pm);

                    if (memoKey != null) {
                        memoModel.delete(loginTicketKey, memoKey);
                    }
                }

                tx.commit();
            } finally {
                if (tx.isActive()) {
                    tx.rollback();
                }
            }

            resp.sendRedirect(this.getServletContext().getContextPath() + "/html/user/myPage.servlet");
        } finally {
            pm.close();
        }
    }

}
